/*******************************************************************
 *
 *	File:		CFileReader.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __C_FILE_READER_H
#define __C_FILE_READER_H

#include "CFileReaderBase.h"


#ifdef COMPRESSED_IMAGE_FILE

#include <ezlib.h>

const TInt KMaxReadBuf = 64 * 1024;

class TGzipReader
{
public:
	void Construct(RFile& aFile);
	void Inflate(RFile& aFile, TDes8& aBuf);
	void CheckEnd();

protected:
	void ReadFile(RFile& aFile);

	TBuf8<KMaxReadBuf>	iReadBuf;
	z_stream			iZLibStream;
};

class CGZipPEFileReader : public CFileReaderBase
{
public:
	~CGZipPEFileReader();
	static CGZipPEFileReader*	NewL();

	virtual void LoadHeadersL(TImageInfo& aImageInfo, TSectionTableEntry* aSectionTable);
	virtual void LoadSectionsL(const TSectionTableEntry* aSectionTable);

protected:	
	void ConstructL();
	CGZipPEFileReader(){};

	void OpenImageL();
	void GunzipSectionL(const TSectionTableEntry& aSectionEntry);

	TGzipReader			iGzipReader;
};

#else

class CPEFileReader : public CFileReaderBase
{
public:
	~CPEFileReader();
	static CPEFileReader*	NewL();

	virtual void LoadHeadersL(TImageInfo& aImageInfo, TSectionTableEntry* aSectionTable);
	virtual void LoadSectionsL(const TSectionTableEntry* aSectionTable);

protected:
	void ConstructL();
	void BuildSectionTableEntryL(const external_scnhdr& aExternalSection, TSectionTableEntry& aSectionTableEntry);

	PEAOUTHDR				iPeOutHeader;
	external_PE_filehdr		iExtPeHeader;
	external_scnhdr			iSectionHeaders[KMaxSectionHeaders];
	TInt					iNoOfSections;
	void					BuildSectionTableL(TSectionTableEntry* aSectionTable);
	TInt					iSectionHeadersIndex[KMaxSectionHeaders];
	external_scnhdr*		Section(TSectionIndex aSectionIndex);

private:

#ifdef WANT_DPRINT
	void DumpHeaders();

#define DPRINT_DUMP_HEADERS() DumpHeaders();

#else

#define DPRINT_DUMP_HEADERS() /**/
#endif
};

#endif

#endif			/* __C_FILE_READER_H */
