/*******************************************************************
 *
 *	File:		CImageLoader.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __C_IMAGE_LOADER_H
#define __C_IMAGE_LOADER_H

#include "CFileReaderBase.h"

/*
 * The following 2 structs are defined based on info from the WIN32 SDK section:
 * "Microsoft Portable Executable and Common Object File Format Specification"
 */
typedef struct
{
	unsigned idte_hint_addr;
	unsigned idte_time_stamp;
	unsigned idte_forward_chain;
	unsigned idte_dll_name;
	unsigned idte_first_thunk;
} import_dir_table_entry;


typedef struct
{
	unsigned page;
	unsigned block_size;
} fixup_block;



class CImageLoader : public CBase
{
public:
	~CImageLoader();
	static CImageLoader* NewLC();
	TUint LoadImageL();

protected:
	void ConstructL();
	void CreateChunksL();
	void ImportLibrariesL();
	void RelocateL();

	void ImportLibraryL(const TDesC& aDllName, const TUidType& aUidType, const import_dir_table_entry* aIdte);
	void GetPureDllNameAndUid(const TDesC& aMangledDllName, TDes& aDllName, TUid& aUid);
	TUint32 ImageVmaToLoadedVma(TUint32 aImageVma);

	TFileName DirectoryImRunningFrom();

	CFileReaderBase*	iFileReader;

	TSectionTableEntry	iSectionTable[KMaxSections];
	TImageInfo			iImageInfo;

	RChunk	iCodeChunk;
	RChunk	iDataChunk;

	HBufC8*		iRelocBuf;
#ifdef WANT_DPRINT
#define DUMP_HEADERS() DumpHeaders()
	void DumpSection(const TSectionTableEntry& aSection);
	void DumpHeaders();
#else
#define DUMP_HEADERS() /**/
#endif

};

inline TUint32 CImageLoader::ImageVmaToLoadedVma(TUint32 aImageVma)
{
	TSectionTableEntry*		entry = &iSectionTable[0];
	for (TInt i=0 ; i< KMaxSections ; i++, entry++)
	{
		if ((aImageVma >= (TUint32) entry->iImageVma) && (aImageVma < (TUint32) (entry->iImageVma + entry->iSize)))
			return (TUint32) entry->iLoadedVma + (aImageVma - (TUint32) entry->iImageVma);
	}
	DPRINT((HERE("CImageLoader::ImageVmaToLoadedVma, could not resolve: 0x%8x"), aImageVma));
	User::Leave(KErrNotSupported);
	return 0;
}


#endif			/* __C_IMAGE_LOADER_H */
