#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>
#include <eikenv.h>
#include <eikdialg.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <eikdlgtb.h>
#include <eiklabel.h>
#include <eikchlst.h>
#include <eikchkbx.h>

#include <bttypes.h>

#include <hello.rsg>
#include "hello.hrh"

#include "ECompXL.h"		// for global C++ object support


#ifdef __HELLO1__
const TUid KUidHelloApp = { 0x101F6948};
#else
const TUid KUidHelloApp = { 0x101F6947};
#endif

#define TLS_MAGIC ((TAny*)	0xdeadcafe)

class TTest
{
public:
	TTest(TInt aData);
	~TTest();
	TInt	iData;
};

TTest::TTest(TInt aData)
{
	iData = aData;
	RDebug::Print(_L("TTest::TTest iData=%d"), iData);
}

TTest::~TTest()
{
	RDebug::Print(_L("TTest::~TTest iData=%d"), iData);
}


#ifndef __WINS__
TTest		test1(1);
TTest		test2(2);
TTest		test3(3);
TTest		test4(4);
TTest		test5(5);
#endif


class TStringTest
{
public:
	TStringTest(const TDesC& aString);
	~TStringTest();
	TBuf<32>	iString;
};

TStringTest::TStringTest(const TDesC& aString)
{
	iString = aString;
	RDebug::Print(_L("TStringTest::TStringTest iString=%S"), &iString);
}

TStringTest::~TStringTest()
{
	RDebug::Print(_L("TStringTest::~TStringTest iString=%S"), &iString);
}

#ifndef __WINS__
TStringTest		stringTest1(_L("String 1"));
TStringTest		stringTest2(_L("String 2"));
TStringTest		stringTest3(_L("String 3"));
TStringTest		stringTest4(_L("String 4"));
TStringTest		stringTest5(_L("String 5"));
#endif

char	sHelloWorld[1024];


class CHelloDocument : public CEikDocument
{
public:
	~CHelloDocument();
	CHelloDocument(CEikApplication& aApp);
	void ConstructL();
private: // from CEikDocument
	CEikAppUi* CreateAppUiL();
};


class CHelloAppView : public CCoeControl, public MCoeControlBrushContext
{
public:
	~CHelloAppView();
	void ConstructL(const TRect& aRect);
private:
	void Draw(const TRect& /*aRect*/) const;
};


class CHelloAppUi : public CEikAppUi
{
public:
	CHelloAppUi(){}
	void ConstructL();
	~CHelloAppUi();

		// cannot be called during construction of the document!
	static CHelloAppUi*	Static() {return (CHelloAppUi*) CEikonEnv::Static()->AppUi();}

protected: // from CEikAppUi

	void HandleCommandL(TInt aCommand);

private:
	CHelloAppView* iAppView;
};


class CHelloApplication : public CEikApplication
	{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	TECompXL	iECompXL;		// support for global C++ objects 
	};


CApaDocument* CHelloApplication::CreateDocumentL()
{
	CHelloDocument*	doc = new (ELeave) CHelloDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CHelloDocument::CHelloDocument(CEikApplication& aApp) : CEikDocument(aApp)
{
}

CHelloDocument::~CHelloDocument()
{
}

void CHelloDocument::ConstructL()
{
}


CEikAppUi* CHelloDocument::CreateAppUiL()
{
	return new(ELeave) CHelloAppUi;
}


CHelloAppUi::~CHelloAppUi()
{
	delete iAppView;
}


void CHelloAppUi::ConstructL()
{

	BaseConstructL();
	iAppView=new(ELeave) CHelloAppView;
	iAppView->ConstructL(ClientRect());
}



void CHelloAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EEikCmdExit: 
			User::InfoPrint(_L("Bye bye..."));
			Exit();
			break;

		case EEikCmdMenuOne:
			User::InfoPrint(_L("That's one"));
			sHelloWorld[0] = 0;
			break;

		case EEikCmdMenuTwo:
			{
				User::InfoPrint(_L("That's two"));

				// test ability to link against bluetooth.dll which doesn't use UIDs
				volatile TBTDevAddr	bTDevAddr;
			}
			break;
		case EEikCmdMenuThree:
		{
			TFileName	name;
			Dll::FileName(name);
			User::InfoPrint(name);
			break;
		}
		case EEikCmdMenuFour:
		{
			if (Dll::Tls())
			{
				User::InfoPrint(_L("Failed: Tls() not initially NULL"));
				break;
			}
			if (Dll::SetTls(TLS_MAGIC) != KErrNone)
			{
				User::InfoPrint(_L("Failed: SetTls() failed"));
				break;
			}
			if (Dll::Tls() != TLS_MAGIC)
			{
				User::InfoPrint(_L("Failed: Tls() not equal to MAGIC value"));
				break;
			}
			if (Dll::SetTls(NULL) != KErrNone)
			{
				User::InfoPrint(_L("Failed: Tls() could not be reset"));
				break;
			}
			User::InfoPrint(_L("All Tls tests succeeded"));
			break;
		}
	}
}


void CHelloAppView::ConstructL(const TRect& aRect)
{
	RDebug::Print(_L("EMAMEAPP: CHelloAppView::ConstructL - start"));

	CreateWindowL();
	SetRect(aRect);
	ActivateL();
}

CHelloAppView::~CHelloAppView()
{
}


void CHelloAppView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();

#ifdef __HELLO1__
	gc.SetBrushColor(KRgbBlue);
	gc.SetPenColor(KRgbBlue);
#else
	gc.SetBrushColor(KRgbRed);
	gc.SetPenColor(KRgbRed);
#endif
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);

	gc.DrawRect(aRect);
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CHelloApplication;
}



TUid CHelloApplication::AppDllUid() const
{
	return KUidHelloApp;
}

